function [W0,BETA0,gamma0,alpha0] = proDSini(X,S,Init,nq,crisp);

% Copyright Thierry Denoeux 
% February 14, 2001
%
% Initialisation of Belief Structure Neural Network
%
% [W0,BETA0,gamma0,alpha0] = proDSini(X,S,Init,nq,crisp);
%
% Input :
%
% nq   : number of prototypes 
% Init : initialisation (1 : nq prototypes ; 
%                        2 : nq prototypes *in each class*)
% X    : data matrix (nb samples x p)
% S    : class labels  (nb samples x 1) ; maximum = M = number of classes 
% crisp : if crisp=1 and Init=2, each prototype has full membership to only 
%         one class (default=0)        
%
% Output :
%
% W0 : n prototypes, size (n x p)   
% BETA0 : parameter used to compute membership degrees, size (n,M)  
% gamma0 : defines influence regions; size = (n,1)  
% alpha0 : size (n,1)
%
% Reference:
%
% T. Denoeux. A neural network classifier based on Dempster-Shafer theory.
% IEEE Trans. on Systems, Man and Cybernetics A, 30(2):131-150, 2000. 

if nargin == 4,
  crisp=0;
end;

M = max(S);
N = size(X,1);
Id = eye(M);
t=Id(S,:);

if Init==2,                   % Initialisation aleatoire                        
 W0=[];
 BETA0=[];
 for i=1:M,
   ii=find(S==i);Ni = length(ii);
   [L,w] = kmean(X(ii,:),nq);
   W0 = [W0;w];  
   BETA0 = [BETA0;t(ii(1:nq),:)];
  end;  
  n=nq*M;
  if ~crisp,
     BETA0 = BETA0 + 0.1*ones(size(BETA0));
  end;
else
  n=nq;
  [class,W0] = kmean(X,n);
  for i=1:n,
    ii = find(class == i);
    if length(ii)==1,
      BETA0(i,:) = sqrt(t(ii,:));
    elseif length(ii)>0,
        BETA0(i,:) = sqrt(mean(t(ii,:)));
    else,
        BETA0(i,:) = rand(1,M);
    end;
   end;
end;


alpha0 = zeros(n,1);
gamma0=0.1*ones(n,1);
