function mk = proDSval(X,W,BETA,ggamma,alpha,inorm);

% Copyright Thierry Denoeux 
% February 14, 2001
%
% Propagation in Belief Structure Neural Network
%
% mk = proDSval(X,W,BETA,ggamma,alpha,inorm);
%
% Input :
%
% X    : data matrix (N x p), N = number of samples, p =input dimension  
% W  : n prototypes, size (n x p)   
% BETA : parameter used to compute membership degrees, size (n,nb of classes)  
% ggamma : defines influence regions; size = (n,1)  
% alpha : size (n,1)
% inorm = 1 if normalization, 0 otherwise (default=1)
%
% Output :
%
% mk = N x (M+1) matrix of belief structures (The first M columns contain the 
% masses given to the singletons; the last column contains m(Omega))
%
% Reference:
%
% T. Denoeux. A neural network classifier based on Dempster-Shafer theory.
% IEEE Trans. on Systems, Man and Cybernetics A, 30(2):131-150, 2000.

if nargin == 5,
  inorm=1;
end;

[n,p]=size(W);
N = size(X,1);
M = size(BETA,2);

BETA2 = BETA .^2;
beta2 = sum(BETA2')';
U = BETA2 ./ (beta2*ones(1,M));
alphap=0.99 ./ (1 + exp(-alpha));
%-----------------------------------------------------------------
% calcul du taux d'erreur
%-----------------------------------------------------------------
mk = [zeros(M,N);ones(1,N)];
for k=1:n,
  d = 0.5*sum((X' - W(k,:)'*ones(1,N)).^2);
  s = alphap(k)*exp(- ggamma(k).^2 * d);
  m = [U(k,:)'*s ; 1-s];
  mk = [ (mk(1:M,:).*(m(1:M,:) + ones(M,1)*m(M+1,:)) + ...
m(1:M,:).*(ones(M,1)*mk(M+1,:)) ) ; (mk(M+1,:).*m(M+1,:))]; 
  end;

if inorm, 
  mk = mk ./(ones(M+1,1)*sum(mk));
end;

mk=mk';


